@@ -53,8 +53,47 @@ correlation: dict = {
 }
 
 
+def has_token(func):
+    def _decorator(self, *args, **kwargs):
+        if self._token is None:
+            raise NoToken("You need to pass in a Discord OAuth2 Token into the object constructor to use this function")
+        return func(self, *args, **kwargs)
+
+    return _decorator
+
+
+def validate(link_type: str) -> bool:
+    valid: list = ["server", "bot", "template"]
+    return link_type.lower() in valid
+
+
+def validate_link_type(func):
+    def _decorator(self, *args, **kwargs):
+        if len(args) > 0:
+            if validate(args[2]):
+                return func(self, *args, **kwargs)
+        else:
+            link = kwargs.get("link_type", None)
+            if link is not None and validate(link):
+                return func(self, *args, **kwargs)
+        raise BadLinkType("link_type must be either 'bot', 'server' or 'template'")
+
+    return _decorator
+
+
+def transfer_ratelimit(func):
+    def _decorator(self, *args, **kwargs):
+        if self.transfer_is_ratelimited:
+            self._v(f"Cannot transfer link - ratelimited.")
+            return
+        return func(self, *args, **kwargs)
+
+    return _decorator
+
+
 class Client:
-    def __init__(self, token: str = None, bearer: bool = False, verbose: bool = False, connection_timeout: int = 15):
+    def __init__(self, token: str = None, bearer: bool = False, verbose: bool = False,
+                 connection_timeout: Union[int, float, None] = 15):
         if token is not None:
             self._token: str = token.strip()
         else:
@@ -68,21 +107,17 @@ class Client:
         if self._bearer and self._token[:7] != "Bearer ":
             self._token: str = "Bearer " + self._token
 
-        self.connection_timeout: int = round(connection_timeout)
+        self.connection_timeout: Union[int, float, None] = connection_timeout
         self._loop: AbstractEventLoop = get_event_loop()
         self._ses: ClientSession = ClientSession(
             loop=self._loop,
-            timeout=ClientTimeout(total=self.connection_timeout))
+            timeout=ClientTimeout(
+                total=self.connection_timeout if self.connection_timeout >= 0 or self.connection_timeout is None else 15))
         self.transfer_is_ratelimited: bool = False
         self._verbose: bool = verbose
 
     @staticmethod
-    def _validate_link_type(link_type: str) -> bool:
-        valid: list = ["server", "bot", "template"]
-        return link_type.lower() in valid
-
-    @staticmethod
-    def _error_from_status(code: int) -> Any:
+    def _raise_from_status(code: int) -> Any:
         return correlation.get(code, None)
 
     @staticmethod
@@ -102,7 +137,9 @@ class Client:
         res = await self._ses.get(f"https://dsc.gg/api/link/{self._format(link)}")
 
         if res.status == 200:
-            return Link(dict(await res.json()))
+            data = dict(await res.json())
+            data["link"] = link
+            return Link(data=data)
 
         self._v(f"Couldn't fetch link '{link}'")
         return None
@@ -111,7 +148,7 @@ class Client:
         res = await self._ses.get(f"https://dsc.gg/api/links/{user_id}")
 
         if res.status == 200:
-            return [Link(dict(link)) for link in list(await res.json())]
+            return [Link(data=dict(link)) for link in list(await res.json())]
 
         self._v(f"Couldn't fetch links for user ID '{user_id}', returning an empty list")
         return []
@@ -120,7 +157,7 @@ class Client:
         res = await self._ses.get("https://dsc.gg/api/top-links")
 
         if res.status == 200:
-            return [Link(dict(link)) for link in list(await res.json())]
+            return [Link(data=dict(link)) for link in list(await res.json())]
 
         self._v("Couldn't fetch top links, returning an empty list")
         return []
@@ -129,7 +166,7 @@ class Client:
         res = await self._ses.get(f"https://dsc.gg/api/all-links?page={page if page > 0 else 1}")
 
         if res.status == 200 and (_ := await res.text()).lower() != "none":
-            return [Link(dict(link)) for link in list(await res.json())]
+            return [Link(data=dict(link)) for link in list(await res.json())]
 
         self._v("Couldn't fetch links, returning an empty list")
         return []
@@ -138,7 +175,7 @@ class Client:
         res = await self._ses.get(f"https://dsc.gg/api/info/{user_id}")
 
         if res.status == 200:
-            return User(dict(await res.json()))
+            return User(data=dict(await res.json()))
 
         self._v(f"Couldn't fetch user with the ID '{user_id}'")
         return None
@@ -147,79 +184,64 @@ class Client:
         res = await self._ses.get(f"https://dsc.gg/api/announcements/{user_id}")
 
         if res.status == 200:
-            return list([Announcement(dict(a)) for a in list(await res.json())])
+            return list([Announcement(data=dict(a)) for a in list(await res.json())])
 
-        self._v(f"Couldn't fetch announcement for user ID '{user_id}'")
+        self._v(f"Couldn't fetch announcements for user ID '{user_id}'")
         return []
 
+    @has_token
+    @validate_link_type
     async def create_link(self, link: Union[str, int], redirect: str, link_type: str) -> NoReturn:
-        self._has_token()
-
-        if not self._validate_link_type(link_type):
-            raise BadLinkType("link_type must be either 'bot', 'server' or 'template'")
-
         res = await self._ses.post("https://dsc.gg/api/create",
                                    headers={"contentType": "application/json"},
                                    json={"link": self._format(link), "redirect": redirect, "type": link_type.lower(),
                                          "token": self._token})
         if res.status != 200:
-            if (err := self._error_from_status(res.status)) is not None:
+            if (err := self._raise_from_status(res.status)) is not None:
                 raise err(correlation.get(err))
 
         self._v(f"Link '{link}' created.")
 
+    @has_token
+    @validate_link_type
     async def update_link(self, link: Union[str, int], redirect: str, link_type: str) -> NoReturn:
-        self._has_token()
-
-        if not self._validate_link_type(link_type):
-            raise BadLinkType("link_type must be either 'bot', 'server' or 'template'")
-
         res = await self._ses.post("https://dsc.gg/api/update",
                                    headers={"contentType": "application/json"},
                                    json={"link": self._format(link), "redirect": redirect, "type": link_type.lower(),
                                          "token": self._token})
 
         if res.status != 200:
-            if (err := self._error_from_status(res.status)) is not None:
+            if (err := self._raise_from_status(res.status)) is not None:
                 raise err(correlation.get(err))
 
         self._v(f"Link '{link}' updated.")
 
+    @has_token
     async def delete_link(self, link: Union[str, int]) -> NoReturn:
-        self._has_token()
-
         res = await self._ses.post("https://dsc.gg/api/delete",
                                    headers={"contentType": "application/json"},
                                    json={"link": self._format(link), "token": self._token})
 
         if res.status != 200:
-            if (err := self._error_from_status(res.status)) is not None:
+            if (err := self._raise_from_status(res.status)) is not None:
                 raise err(correlation.get(err))
 
         self._v(f"Link '{link}' deleted.")
 
+    @has_token
+    @transfer_ratelimit
     async def transfer_link(self, link: Union[str, int], user_id: Union[str, int], comments: str = "None") -> NoReturn:
-        if self.transfer_is_ratelimited:
-            self._v(f"Cannot transfer link '{link}' to user ID '{user_id}' - ratelimited.")
-            return
-
-        self._has_token()
-
         res = await self._ses.post("https://dsc.gg/api/transfer",
                                    json={"link": self._format(link), "comments": comments, "transfer": str(user_id),
                                          "token": self._token}, headers={"contentType": "application/json"})
 
         if res.status != 200:
-            if (err := self._error_from_status(res.status)) is not None:
+            if (err := self._raise_from_status(res.status)) is not None:
                 raise err(correlation.get(err))
         else:
             self._v(f"Successful transfer to user ID '{user_id}' - 5 minute ratelimit")
             self._loop.create_task(self._rate_limit_transfer())
 
-    def _has_token(self) -> NoReturn:
-        if self._token is None:
-            raise NoToken("You need to pass in a Discord OAuth2 Token into the object constructor to use this function")
-
     def _v(self, msg: str) -> NoReturn:
         if self._verbose:
             print(msg)
